(function($) {

/**
 * Where extensions to AUI that aren't ready to migrate should be stored.
 *
 * Functions added here should have a @since version and go on the roadmap to
 * be moved over after a suitable delay.
 */

/**
 * AJS.Data will be the namespace for accessing dynamic data passed from the
 * server to JavaScript via the page HTML.
 *
 * @since 3.4
 */
AJS.Data = $.extend({}, AJS.Data, {

    /**
     * Returns a value given a key. If no entry exists with the key, undefined is returned.
     *
     * @method get
     * @param key
     * @return {String}
     */
    get: function (key) {
        var metaEl = $("meta[name=ajs-" + key + "]");
        return metaEl.length ? metaEl.attr("content") : undefined;
    },

    /**
     * Returns true if the value for the provided key is equal to "true", else returns false.
     * otherwise.
     *
     * @method getBoolean
     * @param key
     * @return {boolean}
     */
    getBoolean: function (key) {
        return this.get(key) === "true";
    }

});

    AJS.logError = function(prefix, ex) {
        // Handle IE and Mozilla style Error objects
        // see https://developer.mozilla.org/en/JavaScript/Reference/Global_Objects/Error
        // see http://msdn.microsoft.com/en-us/library/dww52sbt%28v=vs.94%29.aspx
        var msgs = [];
        for (prop in ex) {
            msgs.push(prop + ': ' + ex[prop]);
        }
        AJS.log(prefix + msgs.join(', '));
    };

    // Override of atlassian.js toInit to add improved exception logging
    AJS.toInit = function (func) {
        $(function () {
            try {
                func.apply(this, arguments);
            } catch(ex) {
                // just calling AJS.log with an Error object gives "[object Error]" in IE,
                // We want better, thus more plumbing for better Error object logging
                AJS.logError("Failed to run init function: ", ex);
            }
        });
        return this;
    };

// should logging appear in the DOM?
if (AJS.Data.getBoolean('log-rendered')) {
    var oldLog = AJS.log,
        logDiv = $('<div id="ajs-log" class="log"><h3>AJS Log</h3>\n</div>'),
        head = $('head'),
        body;

    // should logging appear on screen?
    logDiv.toggleClass('hidden', !AJS.Data.getBoolean('log-visible'));

    // AUI Logging is disabled if there is no console
    // we want to always log to a div as well
    AJS.log = function(obj) {
        var str = (typeof(obj) === "undefined") ? 'undefined' : obj;
        if (body) {
            // ensure the div is the last on the page
            if (logDiv.next().length != 0) {
                body.append(logDiv);
            }
        } else {
            // IE will reject your puny attempts to use jQuery's .text() method here, complaining about an 'Unexpected method or property access'
            // see http://forum.jquery.com/topic/unexpected-call-to-method-or-property-access-in-ie8
            // Using primitive DOM createElement instead
            var script = document.createElement('script');
            script.type = 'text/x-log';
            script.text = str;
            head.append(script);
        }
        logDiv.append($('<p></p>').text('\n' + str));
        oldLog(obj);
    };

    AJS.toInit(function() {
        body = $('body');
        body.append(logDiv);
    });
}

})(AJS.$);
